/*!
* Copyright Adaptavist 2022 (c) All rights reserved
*/
import { GenericAppApiCore } from '@managed-api/generic-core';
import { Headers } from '@managed-api/commons-core';
export class GenericAppApi extends GenericAppApiCore {
    constructor(connectionId) {
        super();
        this.connectionId = connectionId;
    }
    getPlatformImplementation() {
        const decodeInChunks = (input, chunkSize = 1048576) => {
            const decoder = new TextDecoder();
            let decodedString = '';
            let offset = 0;
            while (offset < input.length) {
                const chunk = input.subarray(offset, offset + chunkSize);
                decodedString += decoder.decode(chunk, { stream: true });
                offset += chunkSize;
            }
            decodedString += decoder.decode(new Uint8Array());
            return decodedString;
        };
        return {
            buffer: {
                encode: (input) => new TextEncoder().encode(input),
                decode: (input) => decodeInChunks(new Uint8Array(input))
            },
            performHttpCall: (request) => this.performHttpCall(request)
        };
    }
    async performHttpCall(request) {
        var _a;
        const requestUrl = `${request.url}`;
        request.headers.set('x-stitch-connection-id', this.connectionId);
        const requestHeaders = Object.entries(request.headers.raw()).reduce((acc, [key, value]) => ({ ...acc, [key]: value.join(', ') }), {});
        const response = await fetch(requestUrl, {
            method: request.method,
            headers: requestHeaders,
            body: request.method.toLowerCase() !== 'get' && request.method.toLowerCase() !== 'head' ? ((_a = await request.arrayBuffer()) !== null && _a !== void 0 ? _a : new ArrayBuffer(0)) : undefined
        });
        // TODO: fix incompatibilities so headers could be passed directly
        const apiHeaders = new Headers();
        response.headers.forEach((value, key) => apiHeaders.append(key, value));
        const body = await response.arrayBuffer();
        return super.buildResponse(requestUrl, response.status, response.statusText, apiHeaders, body);
    }
}
